]> git.sesse.net Git - cubemap/blobdiff - stream.h
Use C++11 std::mutex and std::lock_guard instead of our RAII wrapper.
[cubemap] / stream.h
index 4947c3fbd7fdf11a43b8456a78f4a0c80812c58f..17df7f35dade3fc17b577e481031af24ffe2eb8c 100644 (file)
--- a/stream.h
+++ b/stream.h
@@ -8,22 +8,19 @@
 #include <stdint.h>
 #include <sys/types.h>
 #include <sys/uio.h>
+#include <deque>
+#include <mutex>
 #include <string>
 #include <vector>
 
 class StreamProto;
 struct Client;
 
-enum StreamStartSuitability {
-       NOT_SUITABLE_FOR_STREAM_START,
-       SUITABLE_FOR_STREAM_START,
-};
-
 struct Stream {
        // Must be in sync with StreamConfig::Encoding.
        enum Encoding { STREAM_ENCODING_RAW = 0, STREAM_ENCODING_METACUBE };
 
-       Stream(const std::string &stream_id, size_t backlog_size, size_t prebuffering_bytes, Encoding encoding);
+       Stream(const std::string &stream_id, size_t backlog_size, size_t prebuffering_bytes, Encoding encoding, Encoding src_encoding);
        ~Stream();
 
        // Serialization/deserialization.
@@ -36,7 +33,7 @@ struct Stream {
        // Mutex protecting <queued_data> and <queued_data_last_starting_point>.
        // Note that if you want to hold both this and the owning server's
        // <mutex> you will need to take <mutex> before this one.
-       mutable pthread_mutex_t queued_data_mutex;
+       mutable std::mutex queued_data_mutex;
 
        std::string url;
 
@@ -50,6 +47,9 @@ struct Stream {
        // be Metacube, for reflecting to another Cubemap instance).
        Encoding encoding;
 
+       // What encoding we expect the incoming data to be in (usually Metacube).
+       Encoding src_encoding;
+
        // The stream data itself, stored in a circular buffer.
        //
        // We store our data in a file, so that we can send the data to the
@@ -73,11 +73,11 @@ struct Stream {
 
        // How many bytes this stream have received. Can very well be larger
        // than <backlog_size>, since the buffer wraps.
-       size_t bytes_received;
+       size_t bytes_received = 0;
 
-       // The last point in the stream that is suitable to start new clients at
-       // (after having sent the header). -1 if no such point exists yet.
-       ssize_t last_suitable_starting_point;
+       // A list of points in the stream that is suitable to start new clients at
+       // (after having sent the header). Empty if no such point exists yet.
+       std::deque<size_t> suitable_starting_points;
        
        // Clients that are in SENDING_DATA, but that we don't listen on,
        // because we currently don't have any data for them.
@@ -89,15 +89,15 @@ struct Stream {
        std::vector<Client *> to_process;
 
        // Maximum pacing rate for the stream.
-       uint32_t pacing_rate;
+       uint32_t pacing_rate = ~0U;
 
        // Queued data, if any. Protected by <queued_data_mutex>.
        // The data pointers in the iovec are owned by us.
-       std::vector<iovec> queued_data;
-
-       // Index of the last element in queued_data that is suitable to start streaming at.
-       // -1 if none. Protected by <queued_data_mutex>.
-       int queued_data_last_starting_point;
+       struct DataElement {
+               iovec data;
+               uint16_t metacube_flags;
+       };
+       std::vector<DataElement> queued_data;
 
        // Put client to sleep, since there is no more data for it; we will on
        // longer listen on POLLOUT until we get more data. Also, it will be put
@@ -106,7 +106,7 @@ struct Stream {
 
        // Add more data to <queued_data>, adding Metacube headers if needed.
        // Does not take ownership of <data>.
-       void add_data_deferred(const char *data, size_t bytes, StreamStartSuitability suitable_for_stream_start);
+       void add_data_deferred(const char *data, size_t bytes, uint16_t metacube_flags);
 
        // Add queued data to the stream, if any.
        // You should hold the owning Server's <mutex>.
@@ -117,8 +117,14 @@ private:
 
        // Adds data directly to the stream file descriptor, without adding headers or
        // going through <queued_data>.
+       // You should hold the owning Server's <mutex>, and probably call
+       // remove_obsolete_starting_points() afterwards.
+       void add_data_raw(const std::vector<DataElement> &data);
+
+       // Remove points from <suitable_starting_points> that are no longer
+       // in the backlog.
        // You should hold the owning Server's <mutex>.
-       void add_data_raw(const std::vector<iovec> &data);
+       void remove_obsolete_starting_points();
 };
 
 #endif  // !defined(_STREAM_H)